from tinydb import TinyDB, where
from typing import Dict
import json
from os import path


class dbs:

    db = None
    filename = None
    db_name = 'data'
    table = '_default'
    new_data = {}  # type: Dict[str, bool]

    def __init__(self, filename = None):
        if filename:
            self.filename = filename

    def open(self, filename = ''):
        if filename:
            self.filename = filename
        try:
            self.db = TinyDB(self.filename)
            self.new_data[self.table] = True
        except FileNotFoundError:
            self.db = None
            self.filename = ''

    def valid_format(self, filename):
        try:
            if path.getsize(filename) == 0:
                return(True)
            with open(filename) as f:
                json.load(f)
            return True
        except ValueError:
            return False
        except FileNotFoundError:
            return True

    def close(self):
        if self.db is not None:
            self.db.close()
            self.db = None
            self.filename = None

    def status(self):
        if self.filename:
            txt = '\nThe JSON data file is %s.\n\n' % self.filename
            if self.db is not None:
                if len(self.db.tables()) > 0:
                    txt += 'There are %s tables in this data file as follows:\n' % len(self.db.tables())
                    total_records = 0
                    for table_name in self.db.tables():
                        total_records += len(self.db.table(table_name))
                        txt += "  A table named '%s' with %s records.\n" % (table_name, len(self.db.table(table_name)))
                    txt += '  In total there are %s records in the data file.\n' % total_records
                    txt += "The current table is '%s'.\n" % self.table
                else:
                    txt += 'There are no tables in this data file.\n'
            else:
                txt += 'The data file is empty or has not been initialized.\n'
        else:
            txt = '\nA data file has not been opened.\n'
        return(txt)

    def fields(self, tablename = ''):
        fieldnames = []
        for row in self.db.table(tablename if tablename else self.table):
            for fieldname in row.keys():
                if fieldname not in fieldnames:
                    fieldnames.append(fieldname)
        return(fieldnames)

    def save(self, data_record):
        self.db.table(self.table).insert(data_record)
        self.new_data[self.table] = True
        return(True)

    # def __len__(self):
    #    return(len(self.db) if self.db else 0)

    def names(self, fieldname):
        return([row[fieldname] for row in self.db.table(self.table)])

    def replace(self, data_record):
        pass

    def duplicate(self, data_record):
        pass

    def delete(self, doc_id):
        self.db.table(self.table).remove(doc_ids = [doc_id])
        self.new_data[self.table] = True

    def delete_all(self, table_name = None):
        if self.db is not None:
            if table_name is None:
                self.db.drop_tables()
            else:
                self.db.table(table_name).truncate()

    def get_unitid(self, name):
        unit, idno = name.split('-')
        p = self.db.table(self.table).search((where('unit') == unit.strip()) & (where('id') == idno.strip()))
        if p:
            return(p)
        else:
            return(None)

    def last_record(self):
        try:
            last = self.db.table(self.table).all()[-1]
        except (IndexError, AttributeError):
            last = None
        return(last)

    def doc_ids(self):
        return([r.doc_id for r in self.db.table(self.table).all()])



from os import path
from kivy.utils import platform
from kivy.core.window import Window

# See if the file as given can be found.
# If not, try to find it in the same folder as the CFG file.
# This can happen when a CFG and associated files are copied to a
# new folder or computer or device.


def locate_file(filename, cfg_path = None):
    if path.isfile(filename):
        return(filename)
    if cfg_path:
        p, f = path.split(filename)
        if path.isfile(path.join(cfg_path, f)):
            return(path.join(cfg_path, f))
    return('')


def filename_only(filename = None):
    if filename:
        p, f = path.split(filename)
        return(f)
    else:
        return('')


def platform_name():
    return(['Windows', 'Linux', 'Android', 'MacOSX', 'IOS', 'Unknown'][['win', 'linux', 'android', 'macosx', 'ios', 'unknown'].index(platform)])


def restore_window_size_position(main_name, main_ini):
    Window.minimum_width = 450
    Window.minimum_height = 450
    if main_ini.get_value(main_name, "SCREENTOP"):
        temp = max(int(main_ini.get_value(main_name, "SCREENTOP")), 0)
        Window.top = temp
    if not main_ini.get_value(main_name, "SCREENLEFT") == '':
        temp = max(int(main_ini.get_value(main_name, "SCREENLEFT")), 0)
        Window.left = temp
    window_width = None
    window_height = None
    if not main_ini.get_value(main_name, "SCREENWIDTH") == '':
        window_width = max(int(main_ini.get_value(main_name, "SCREENWIDTH")), 450)
    if not main_ini.get_value(main_name, "SCREENHEIGHT") == '':
        window_height = max(int(main_ini.get_value(main_name, "SCREENHEIGHT")), 450)
    if window_width and window_height:
        Window.size = (window_width, window_height)



# To Do
#   Consider implications of not writing the block data after each update

import logging
import os
from edmpy.lib.constants import __program__


class blockdata:

    filename = ''
    blocks = []

    def update_value(self, blockname, varname, vardata, append = False):
        block_exists = False
        for block in self.blocks:
            if block['BLOCKNAME'] == blockname.upper():
                block_exists = True
                if (varname.upper() in block.keys()) and append:
                    block[varname.upper()] = [block[varname.upper()], vardata]
                else:
                    block[varname.upper()] = vardata
                return(True)
        if not block_exists:
            temp = {}
            temp['BLOCKNAME'] = blockname.upper()
            temp[varname.upper()] = vardata
            self.blocks.append(temp)
            return(True)
        return(False)

    def get_block(self, blockname):
        if self.blocks:
            for block in self.blocks:
                if block['BLOCKNAME'] == blockname.upper():
                    return(block)
        return('')

    def read_blocks(self):
        self.blocks = []
        try:
            if os.path.isfile(self.filename):
                with open(self.filename) as f:
                    for line in f:
                        if len(line) > 2:
                            if line.strip()[0] == "[":
                                blockname = line.strip()[1:-1].upper()
                            else:
                                if '=' in line:
                                    varname = line.split("=")[0].strip().upper()
                                    vardata = line.split("=")[1].strip()
                                    self.update_value(blockname, varname, vardata)
            else:
                f = open(self.filename, mode = 'w')
                f.close()
        except Exception as ex:
            template = "An exception of type {0} occurred. Arguments:\n{1!r}"
            message = template.format(type(ex).__name__, ex.args)
            logging.exception(message)
            print(message)
        return(self.blocks)

    def names(self):
        name_list = []
        for block in self.blocks:
            name_list.append(block['BLOCKNAME'])
        return(name_list)

    def fields(self):
        field_names = [field for field in self.names() if field not in [__program__]]
        return(field_names)

    def get_value(self, blockname, varname):
        if self.blocks:
            for block in self.blocks:
                if block['BLOCKNAME'] == blockname.upper():
                    if varname.upper() in block.keys():
                        return(block[varname.upper()])
                    else:
                        return('')
        return('')

    def delete_key(self, blockname, key):
        for block in self.blocks:
            if block['BLOCKNAME'] == blockname.upper():
                block.pop(key)
                return

    def rename_block(self, oldname, newname):
        for block in self.blocks:
            if block['BLOCKNAME'] == oldname.upper():
                block['BLOCKNAME'] = newname.upper()
                return

    def rename_key(self, blockname, old_key, new_key):
        for block in self.blocks:
            if block['BLOCKNAME'] == blockname.upper():
                block[new_key] = block.pop(old_key)
                return

    def write_blocks(self):
        try:
            with open(self.filename, mode = 'w') as f:
                for block in self.blocks:
                    f.write("[%s]\n" % block['BLOCKNAME'])
                    for item in block.keys():
                        if not item == 'BLOCKNAME' and not item[:2] == "__":
                            if block[item]:
                                f.write(item + "=%s\n" % block[item])
                    f.write("\n")
            return(True)
        except OSError:
            return(False)

    def save(self):
        self.write_blocks()

    def __len__(self):
        return(len(self.blocks))
